
class Parser:
  '''
  A Python parser
  '''
  
  def __init__(self, obj = None, subClasses = None):
    '''
    A Bootstrap class wraps an implementation class (or another bootstrap class). The subclass list is used for virtualizing calls.
    We have to initialize the parent first in case the implementation object makes calls during __init__()
    '''
    self.classes = []
    if obj is None:
      if not subClasses is None:
        self.classes.extend(subClasses)
      import ASE.BaseClass
      self.parent = ASE.BaseClass.BaseClass(subClasses = self.classes)
      import ASE.Compiler.Python.Parser_impl
      self.obj = ASE.Compiler.Python.Parser_impl.Parser(self)
      self.classes.append(self.obj)
      self.parent.classes.insert(-1, self.obj)
    else:
      self.obj = obj
      self.parent = obj
  
  def __getattr__(self, attr):
    for klass in self.classes:
      if hasattr(klass, attr):
        return getattr(klass, attr)
    if not self.parent is None:
      return getattr(self.parent, attr)
    raise AttributeError, attr
  
  def getBaseObject(obj):
    while hasattr(obj, 'obj'):
      obj = obj.obj
    return obj
  getBaseObject = staticmethod(getBaseObject)
  
  def __eq__(self, obj):
    if not hasattr(obj, 'obj'):
      return 0
    return self.isSame(obj)
  
  def __hash__(self):
    return hash(id(self.getBaseObject(self)))
  
  def __nonzero__(self):
    return 1
  
  def __repr__(self):
    return '<class ' + self.getClassName() + ' at ' + hex(id(self)) + '>'
  
  def __str__(self):
    return '<class ' + self.getClassName() + ' at ' + hex(id(self)) + '>'
  
  def isSame(self, obj):
    '''
    Return true if and only if <code>obj</code> refers to the same underlying object as this object
    '''
    return id(self.getBaseObject(self)) == id(self.getBaseObject(obj))
  
  def getClassName(self):
    '''
    Return the type name of the underlying class object
    '''
    return 'ASE.Compiler.Python.Parser'
  
  def isInstanceOf(self, typeName):
    '''
    Return <code>true</code> if this object is an instance of the specified type
    - This routine will return <code>true</code> if and only if a cast to the specified type would succeed
    '''
    if len(self.classes) > 1:
      import ASE.Loader
      return ASE.Loader.Loader.createClass(self.classes[0].__module__[:-5]).isInstanceOf(typeName)
    elif len(self.classes) == 0 and not self.obj is None:
      return self.obj.isInstanceOf(typeName)
    elif typeName in ('ASE.BaseInterface', 'ASE.BaseClass', 'ASE.Compiler.Parser', 'ASE.Compiler.Python.Parser'):
      return 1
    return 0
